from enum import Enum

class {{ name }}(str, Enum):
    {% for enumItem in enum %}

    {% if enumItem is string %}
    {{ enumItem.lower() }} = '{{ enumItem }}'{% else %}
    value_{{ enumItem }} = {{ enumItem }}{% endif %}
    {% endfor %}
